from django.contrib.auth.models import BaseUserManager, AbstractBaseUser
from django.db import models
from django.utils.translation import gettext_lazy as _

from pronounscc.snowflake import Snowflake


class UserManager(BaseUserManager):
    def create_user(
        self,
        username: str,
        email: str | None,
        discord: tuple[int, str] | None,
        password=None,
    ):
        if (not email and not password) and not discord:
            raise ValueError("Users must have an authentication method")

        user = self.model(
            username=username,
            email=self.normalize_email(email) if email else None,
            discord=discord[0] if discord else None,
            discord_username=discord[1] if discord else None,
        )

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, username: str, email: str, password=None):
        user = self.model(
            username=username,
            email=self.normalize_email(email),
        )

        user.set_password(password)
        user.is_admin = True
        user.save(using=self._db)
        return user


class User(AbstractBaseUser):
    """The basic user model that handles authentication.

    User profile data is in UserProfile instead."""

    uid = models.BigIntegerField(_("user id"), primary_key=True, default=Snowflake.generate)
    username = models.CharField(_("username"), unique=True)
    email = models.CharField(_("email"), unique=True, blank=True, null=True)

    discord = models.BigIntegerField(_("discord id"), unique=True, null=True)
    discord_username = models.CharField(_("discord username"), blank=True)

    is_moderator = models.BooleanField(_("is moderator"), default=False)
    is_admin = models.BooleanField(_("is admin"), default=False)

    objects = UserManager()

    USERNAME_FIELD = "username"
    EMAIL_FIELD = "email"
    REQUIRED_FIELDS = ["email"]

    @property
    def is_staff(self):
        return self.is_moderator or self.is_admin

    def has_perm(self, perm, obj=None):
        return self.is_staff

    def has_module_perms(self, app_label):
        return self.is_staff



class UserProfile(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE, primary_key=True)

    display_name = models.CharField(_("display name"), max_length=100, blank=True)
    bio = models.TextField(_("bio"), max_length=1000, blank=True)
